#version 330

uniform vec4 ambientColor;
uniform vec4 diffuseColor;   
uniform vec4 specularColor;
uniform float shininess; 
uniform sampler2D colorMap;
uniform sampler2DShadow shadowTexture;
uniform sampler2D textureUnit2;

in vec3 vVaryingNormal;
in vec3 vVaryingLightDir;
in vec2 vTexCoords;
in vec4 projCoord;

in vec4 curPos;
in vec4 prevPos;

//in vec3 speedInfo;
//in vec2 speed;

layout (location = 0) out vec4 vFragColor;
layout (location = 1) out vec4 speedOut;
layout (location = 2) out vec4 speedInfoOut;

void main(void)
{
	// Dot product gives us diffuse intensity
	float diff = max(0.0, dot(normalize(vVaryingNormal), normalize(vVaryingLightDir)));

	// Multiply intensity by diffuse color, force alpha to 1.0
	vFragColor = diff * diffuseColor;

	// Add in ambient light
	vFragColor += ambientColor;

	// Specular Light
	vec3 vReflection = normalize(reflect(-normalize(vVaryingLightDir), normalize(vVaryingNormal)));
	float spec = max(0.0, dot(normalize(vVaryingNormal), vReflection));

	float shadow = textureProj(shadowTexture, projCoord);

	shadow = shadow * 0.5 + 0.5;

	if(diff > 0)
	{
		float fSpec = pow(spec, shininess);
		if (shadow >= 0.5)
			vFragColor.rgb += vec3(fSpec, fSpec, fSpec);
	}

	// Modulate in the texture
	vFragColor *= texture(colorMap, vTexCoords);

	vFragColor *= shadow;

	vec2 speed = (curPos.xy / curPos.w) - (prevPos.xy / prevPos.w);
	speed *= 2;
	vec3 speedInfo = vec3(normalize(speed), length(speed));

	speedOut = vec4(length(speed), length(speed), length(speed), 1.0);
	speedInfoOut = vec4(0.5 + 0.5 * speedInfo.xy, pow(speedInfo.z, 0.5), 0.0); 
}
    