#version 130
in vec4 vVertex;
in vec3 vNormal;
in vec3 vTangent;
in vec2 vTexture0;

uniform mat4 prevMVPMatrix;
uniform mat4 mvpMatrix;
uniform mat4 mvMatrix;
uniform mat3 normalMatrix;
uniform vec3 vLightPosition;
uniform mat4 lightMatrix;

// Color to fragment program
out vec3 vVaryingLightDir;
out vec2 vTexCoords;

out vec4 projCoord;

out vec4 curPos;
out vec4 prevPos;

void main(void) 
{ 
	vec3 n = normalize(normalMatrix * vNormal);
	vec3 t = normalize(normalMatrix * vTangent);
	vec3 b = cross(t, n);

	vec4 vertexPosition = mvMatrix * vVertex;

	// move light-direction to tangent space
	vec3 tmp = normalize(vLightPosition - (vertexPosition.xyz / vertexPosition.w));
	vVaryingLightDir.x = dot(tmp, t);
	vVaryingLightDir.y = dot(tmp, b);
	vVaryingLightDir.z = dot(tmp, n);

	// Pass along the texture coordinates
	vTexCoords = vTexture0 * 4; // HACK: scale texture

	projCoord = lightMatrix * vVertex;

	curPos = mvpMatrix * vVertex;
	prevPos = prevMVPMatrix * vVertex;

	// Don't forget to transform the geometry!
	gl_Position = mvpMatrix * vVertex;
}
