#version 330

in vec2 vTexCoord;
uniform sampler2DMS colorTexture;
uniform sampler2DMS velocityMap;
uniform float maxSamples;
uniform float intensity;

out vec4 fragColor;

void main(void) 
{ 
	vec2 tmp = floor(textureSize(velocityMap) * vTexCoord);

	// map from [0,1] back to [-1,1]
	vec3 speedInfo = texelFetch(velocityMap, ivec2(tmp), 0).rgb;

	vec2 speed = (2.0 * speedInfo.xy - 1.0) * pow(speedInfo.z, 2.0);
	vec2 offset = intensity * speed;

	vec4 c = vec4(0.0);
	vec4 msColor = vec4(0.0);

	float inc = 0.1;
	float weight = 0.0;

	for(float i = 0.0; i <= 1.0; i += inc)
	{
		tmp = floor(textureSize(colorTexture) * (vTexCoord + (i * offset)));

		// 4x fsaa
		for(int sampleNum = 0; sampleNum <= maxSamples; sampleNum++)
		{
			msColor += texelFetch(colorTexture, ivec2(tmp), sampleNum); 
		} 
		msColor /= (maxSamples + 1);

		c += msColor;

		msColor = vec4(0.0);

		weight += 1.0;
	}
	
	fragColor = c / weight;
}